from copy import copy
from typing import List, Optional

import copy
from app.gun import Gun
from app.armor import Armor, HArmour, SArmour
from app.ammo import (
    Ammo, HEATCartridge,
    HECartridge, APCartridge)


class Tank:
    def __init__(
            self, model: str, gun: Gun, health: int,
            armors: List[Armor] = None, ammos: List[Ammo] = None,
    ) -> None:
        self.model = model
        self.gun = gun
        self.armors = armors if armors else self._init_armours(30)
        self.ammos = ammos if ammos else self._init_ammos(10)
        self.health = health
        self._selected_armour = armors[0]
        self._loaded_ammo = None

    def _init_armours(self, armour_width: int):
        self.armors = [cls(armour_width) for cls in [HArmour, SArmour]]

    def _init_ammos(self, ammos_count=10):
        for cls in [HEATCartridge, HECartridge, APCartridge]:
            for _ in range(ammos_count):
                self.ammos.append(cls(self.gun))

    def select_armour(self, armour_type: str):
        self._selected_armour = list(
            filter(lambda x: x.armour_type == armour_type, self.armors)
        )[0]

    def load_gun(self, ammo_type: str):
        """заряжает пушку снарядом выбранного типа и уменьшает общее количество снарядов этого типа в боеукладке"""

        searched_ammos = list(
            filter(lambda x: x.ammo_type == ammo_type, self.ammos)
        )

        if searched_ammos:
            self._loaded_ammo = searched_ammos[0]
        else:
            raise Exception(f'{ammo_type} not exist')

    def _remove_loaded_ammo(self):
        self.ammos.remove(self._loaded_ammo)
        self._loaded_ammo = None

    def shoot(self) -> Optional[Ammo]:
        if not self._loaded_ammo:
            raise Exception(f'Gun {self.gun} must be loaded!')

        fired_ammo = copy(self._loaded_ammo)
        self._remove_loaded_ammo()

        is_on_target = self.gun.is_on_target()

        if is_on_target:
            print('Попадание')
            return fired_ammo

    def handle_hit(self, ammo: Ammo):
        if self._selected_armour.is_penetrated(ammo):
            self.health -= ammo.get_demage()
        else:
            print('Броня не пробита!')



import gun


class Ammo(object):
    """class Ammo - материнский класс для классов-наследников снарядов
    Класс Ammo имеет следующие поля:
        -gun(каждый снаряд делается под конкретную пушку,
    так что здесь стоит применить агрегацию)
        -type (фугасный, кумулятивный, подкалиберный)
    """

    def __init__(self, gun: gun, shell_type: str):
        self.gun = gun
        self.shell_type = shell_type

    def get_damage(self):
        """метод возвращает калибр пушки, умноженный на 3"""

        return self.gun.caliber * 3

    def get_penetration(self):
        """снаряд может пробить броню, равную по толщине своему калибру
        метод возвращает калибр пушки"""
        return self.gun.caliber


class HECartridge(Ammo):
    """класс фугасных снарядов.
    метод get_damage никак не изменяется"""

    def __init__(self, gun: gun.Gun):
        super().__init__(gun, 'фугасный')


class HEATCartridge(Ammo):
    """класс кумулятивный снаряд"""

    def __init__(self, gun: gun.Gun):
        super().__init__(gun, 'кумулятивный')

    def get_damage(self):
        return super().get_damage() * 0.6


class APCartridge(Ammo):
    """класс подкалиберного снаряда"""

    def __init__(self, gun: gun.Gun):
        super().__init__(gun, 'подкалиберный')

    def get_damage(self):
        return super().get_damage() * 0.3



import ammo


class Armor:
    """thickness (толщина брони)
     type(тип брони)
     """

    def __init__(self, thickness: int, thickness_type: str):
        self.thickness = thickness
        self.type = thickness_type

    def is_penetrated(self, ammo: ammo):
        return True if getattr(ammo.gun, 'caliber') > self.thickness else False


class HArmour(Armor):
    ammo_type_coeff = {
        'фугасный': 1.2,
        'подкалиберный': 1,
        'кумулятивный': 0.7
    }

    def __init__(self, thickness: int):
        super().__init__(thickness, 'гомогенная')

    def is_penetrated(self, ammo: ammo):
        return True if getattr(ammo.gun, 'caliber') > self.thickness * \
                       self.ammo_type_coeff[getattr(ammo, 'shell_type')] else False


class SArmour(Armor):
    """ стальная"""

    ammo_type_coeff = {
        'фугасный': 0.6,
        'подкалиберный': 1.2,
        'кумулятивный': 1.5
    }

    def __init__(self, thickness: int):
        super().__init__(thickness, 'стальная')

    def is_penetrated(self, ammo: ammo):
        return True if getattr(ammo.gun, 'caliber') > self.thickness * \
                       self.ammo_type_coeff[getattr(ammo, 'shell_type')] else False


class CArmour(Armor):
    """керамическая"""

    ammo_type_coeff = {
        'фугасный': 0.6,
        'подкалиберный': 1.2,
        'кумулятивный': 1.5
    }

    def __init__(self, thickness: int):
        super().__init__(thickness, 'керамическая')

    def is_penetrated(self, ammo: ammo):
        return True if getattr(ammo.gun, 'caliber') > self.thickness * \
                       self.ammo_type_coeff[getattr(ammo, 'shell_type')] else False




from setuptools import setup

setup(
    name='bb_krl Tank Battle',
    version='1.0.0',
    packages=['app'],
    url='https://github.com/bbkrl/TMS_homework',
    license='',
    author='bb_krl',
    author_email='bagushevich2003@mail.ru',
    description='package for consol tank battle'
)



from random import triangular


class Gun(object):
    """класс отвечает за калибр и длину ствола.
    От калибра зависит урон, и, частично, способность к пробитию брони.
    От длины ствола зависит точность стрельбы.
    класс имеет метод is_on_target -> bool, отвечающий за попадание
    """

    def __init__(self, caliber, barrel_length):
        self.caliber = caliber
        self.barrel_length = barrel_length

    def is_on_target(self) -> bool:
        """метод принимает(?) случаюную величину
        возвращает результат bool
        формула расчета: ({длинна пушки) *{dice}) > 100
        """

        rand = triangular(1.0, 6.0)
        return True if self.barrel_length * rand > 100 else False
